@@ -69,31 +69,56 @@ static gpointer session_object = NULL;
 
 static void
 on_info (GdmGreeterClient  *client,
+         const char        *service_name,
          const char        *text,
          GdmGreeterSession *session)
 {
         g_debug ("GdmGreeterSession: Info: %s", text);
 
-        gdm_greeter_login_window_info (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window), text);
+        gdm_greeter_login_window_info (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window), service_name, text);
 }
 
 static void
 on_problem (GdmGreeterClient  *client,
+            const char        *service_name,
             const char        *text,
             GdmGreeterSession *session)
 {
         g_debug ("GdmGreeterSession: Problem: %s", text);
 
-        gdm_greeter_login_window_problem (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window), text);
+        gdm_greeter_login_window_problem (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window), service_name, text);
+}
+
+static void
+on_service_unavailable (GdmGreeterClient  *client,
+                        const char        *service_name,
+                        GdmGreeterSession *session)
+{
+        g_debug ("GdmGreeterSession: Service Unavailable: %s", service_name);
+
+        gdm_greeter_login_window_service_unavailable (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window), service_name);
 }
 
 static void
 on_ready (GdmGreeterClient  *client,
+          const char        *service_name,
           GdmGreeterSession *session)
 {
         g_debug ("GdmGreeterSession: Ready");
 
-        gdm_greeter_login_window_ready (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window));
+        gdm_greeter_login_window_ready (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window),
+                                        service_name);
+}
+
+static void
+on_conversation_stopped (GdmGreeterClient  *client,
+                         const char        *service_name,
+                         GdmGreeterSession *session)
+{
+        g_debug ("GdmGreeterSession: Conversation '%s' stopped", service_name);
+
+        gdm_greeter_login_window_conversation_stopped (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window),
+                                                       service_name);
 }
 
 static void
@@ -107,28 +132,6 @@ on_reset (GdmGreeterClient  *client,
         gdm_greeter_login_window_reset (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window));
 }
 
-static void
-on_authentication_failed (GdmGreeterClient  *client,
-                          GdmGreeterSession *session)
-{
-        g_debug ("GdmGreeterSession: Authentication failed");
-
-        session->priv->num_tries++;
-
-        if (session->priv->num_tries < MAX_LOGIN_TRIES) {
-                g_debug ("GdmGreeterSession: Retrying login (%d)",
-                         session->priv->num_tries);
-
-                gdm_greeter_login_window_authentication_failed (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window));
-        } else {
-                g_debug ("GdmGreeterSession: Maximum number of login tries exceeded (%d) - resetting",
-                         session->priv->num_tries - 1);
-                session->priv->num_tries = 0;
-
-                gdm_greeter_login_window_reset (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window));
-        }
-}
-
 static void
 show_or_hide_user_options (GdmGreeterSession *session,
                            const char        *username)
@@ -163,6 +166,7 @@ on_default_session_name_changed (GdmGreeterClient  *client,
                                  GdmGreeterSession *session)
 {
         g_debug ("GdmGreeterSession: default session name changed: %s", text);
+        gdm_greeter_login_window_set_default_session_name (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window), text);
 }
 
 static void
@@ -176,33 +180,44 @@ on_timed_login_requested (GdmGreeterClient  *client,
 }
 
 static void
-on_user_authorized (GdmGreeterClient  *client,
-                    GdmGreeterSession *session)
+on_session_opened (GdmGreeterClient  *client,
+                   const char        *service_name,
+                   GdmGreeterSession *session)
 {
-        g_debug ("GdmGreeterSession: user authorized");
-        gdm_greeter_login_window_user_authorized (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window));
+        g_debug ("GdmGreeterSession: session opened");
+        gdm_greeter_login_window_session_opened (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window), service_name);
 }
 
 static void
 on_info_query (GdmGreeterClient  *client,
+               const char        *service_name,
                const char        *text,
                GdmGreeterSession *session)
 {
         g_debug ("GdmGreeterSession: Info query: %s", text);
 
-        gdm_greeter_login_window_info_query (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window), text);
+        gdm_greeter_login_window_info_query (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window), service_name, text);
 }
 
 static void
 on_secret_info_query (GdmGreeterClient  *client,
+                      const char        *service_name,
                       const char        *text,
                       GdmGreeterSession *session)
 {
         g_debug ("GdmGreeterSession: Secret info query: %s", text);
 
-        gdm_greeter_login_window_secret_info_query (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window), text);
+        gdm_greeter_login_window_secret_info_query (GDM_GREETER_LOGIN_WINDOW (session->priv->login_window), service_name, text);
 }
 
+static void
+on_start_conversation (GdmGreeterLoginWindow *login_window,
+                       const char            *service_name,
+                       GdmGreeterSession     *session)
+{
+        gdm_greeter_client_call_start_conversation (session->priv->client,
+                                                    service_name);
+}
 static void
 on_begin_auto_login (GdmGreeterLoginWindow *login_window,
                      const char            *username,
@@ -214,45 +229,44 @@ on_begin_auto_login (GdmGreeterLoginWindow *login_window,
 
 static void
 on_begin_verification (GdmGreeterLoginWindow *login_window,
+                       const char            *service_name,
                        GdmGreeterSession     *session)
 {
-        gdm_greeter_client_call_begin_verification (session->priv->client);
+        gdm_greeter_client_call_begin_verification (session->priv->client,
+                                                    service_name);
 }
 
 static void
 on_begin_verification_for_user (GdmGreeterLoginWindow *login_window,
+                                const char            *service_name,
                                 const char            *username,
                                 GdmGreeterSession     *session)
 {
         gdm_greeter_client_call_begin_verification_for_user (session->priv->client,
+                                                             service_name,
                                                              username);
 }
 
 static void
 on_query_answer (GdmGreeterLoginWindow *login_window,
+                 const char            *service_name,
                  const char            *text,
                  GdmGreeterSession     *session)
 {
         gdm_greeter_client_call_answer_query (session->priv->client,
+                                              service_name,
                                               text);
 }
 
 static void
-on_select_session (GdmGreeterSession     *session,
-                   const char            *text)
+on_select_session (GdmGreeterLoginWindow *login_window,
+                   const char            *text,
+                   GdmGreeterSession     *session)
 {
         gdm_greeter_client_call_select_session (session->priv->client,
                                                 text);
 }
 
-static void
-on_select_language (GdmGreeterSession     *session,
-                    const char            *text)
-{
-        gdm_greeter_client_call_select_language (session->priv->client,
-                                                 text);
-}
-
 static void
 on_select_user (GdmGreeterLoginWindow *login_window,
                 const char            *text,
@@ -271,17 +285,17 @@ on_cancelled (GdmGreeterLoginWindow *login_window,
 }
 
 static void
-on_disconnected (GdmGreeterLoginWindow *login_window,
-                 GdmGreeterSession     *session)
+on_disconnected (GdmGreeterSession     *session)
 {
         gdm_greeter_client_call_disconnect (session->priv->client);
 }
 
 static void
 on_start_session (GdmGreeterLoginWindow *login_window,
+                  const char            *service_name,
                   GdmGreeterSession     *session)
 {
-        gdm_greeter_client_call_start_session_when_ready (session->priv->client, TRUE);
+        gdm_greeter_client_call_start_session_when_ready (session->priv->client, service_name, TRUE);
 }
 
 static int
@@ -341,13 +355,8 @@ toggle_panel (GdmGreeterSession *session,
                 session->priv->panel = gdm_greeter_panel_new (screen, monitor, is_local);
 
                 g_signal_connect_swapped (session->priv->panel,
-                                          "language-selected",
-                                          G_CALLBACK (on_select_language),
-                                          session);
-
-                g_signal_connect_swapped (session->priv->panel,
-                                          "session-selected",
-                                          G_CALLBACK (on_select_session),
+                                          "disconnected",
+                                          G_CALLBACK (on_disconnected),
                                           session);
 
                 gtk_widget_show (session->priv->panel);
@@ -371,7 +380,10 @@ toggle_login_window (GdmGreeterSession *session,
                 is_local = gdm_greeter_client_get_display_is_local (session->priv->client);
                 g_debug ("GdmGreeterSession: Starting a login window local:%d", is_local);
                 session->priv->login_window = gdm_greeter_login_window_new (is_local);
-
+                g_signal_connect (session->priv->login_window,
+                                  "start-conversation",
+                                  G_CALLBACK (on_start_conversation),
+                                  session);
                 g_signal_connect (session->priv->login_window,
                                   "begin-auto-login",
                                   G_CALLBACK (on_begin_auto_login),
@@ -393,12 +405,12 @@ toggle_login_window (GdmGreeterSession *session,
                                   G_CALLBACK (on_select_user),
                                   session);
                 g_signal_connect (session->priv->login_window,
-                                  "cancelled",
-                                  G_CALLBACK (on_cancelled),
+                                  "session-selected",
+                                  G_CALLBACK (on_select_session),
                                   session);
                 g_signal_connect (session->priv->login_window,
-                                  "disconnected",
-                                  G_CALLBACK (on_disconnected),
+                                  "cancelled",
+                                  G_CALLBACK (on_cancelled),
                                   session);
                 g_signal_connect (session->priv->login_window,
                                   "start-session",
@@ -559,17 +571,21 @@ gdm_greeter_session_init (GdmGreeterSession *session)
                           "problem",
                           G_CALLBACK (on_problem),
                           session);
+        g_signal_connect (session->priv->client,
+                          "service-unavailable",
+                          G_CALLBACK (on_service_unavailable),
+                          session);
         g_signal_connect (session->priv->client,
                           "ready",
                           G_CALLBACK (on_ready),
                           session);
         g_signal_connect (session->priv->client,
-                          "reset",
-                          G_CALLBACK (on_reset),
+                          "conversation-stopped",
+                          G_CALLBACK (on_conversation_stopped),
                           session);
         g_signal_connect (session->priv->client,
-                          "authentication-failed",
-                          G_CALLBACK (on_authentication_failed),
+                          "reset",
+                          G_CALLBACK (on_reset),
                           session);
         g_signal_connect (session->priv->client,
                           "selected-user-changed",
@@ -588,8 +604,8 @@ gdm_greeter_session_init (GdmGreeterSession *session)
                           G_CALLBACK (on_timed_login_requested),
                           session);
         g_signal_connect (session->priv->client,
-                          "user-authorized",
-                          G_CALLBACK (on_user_authorized),
+                          "session-opened",
+                          G_CALLBACK (on_session_opened),
                           session);
 
         /* We want to listen for panel mnemonics even if the
